@@ -1,18 +1,21 @@
-import logging
+import datetime
 import operator
 from typing import Optional
 
 import cachetools
+import elasticsearch
 import numpy as np
+from elasticsearch.helpers import scan as elasticsearch_scan
 
 from robotoff import settings
+from robotoff.elasticsearch import get_es_client
 from robotoff.insights.importer import import_insights
 from robotoff.logo_label_type import LogoLabelType
-from robotoff.models import ImageModel, LogoAnnotation, LogoConfidenceThreshold
+from robotoff.models import LogoAnnotation, LogoConfidenceThreshold, LogoEmbedding
 from robotoff.prediction.types import Prediction
 from robotoff.slack import NotifierFactory
-from robotoff.types import PredictionType
-from robotoff.utils import get_logger, http_session
+from robotoff.types import ElasticSearchIndex, PredictionType
+from robotoff.utils import get_logger
 from robotoff.utils.types import JSONType
 
 logger = get_logger(__name__)
@@ -97,76 +100,73 @@ def get_logo_confidence_thresholds() -> dict[LogoLabelType, float]:
 
 
 def get_stored_logo_ids() -> set[int]:
-    r = http_session.get(
-        settings.BaseURLProvider().robotoff().get() + "/api/v1/ann/stored", timeout=30
+    es_client = get_es_client()
+    scan_iter = elasticsearch_scan(
+        es_client,
+        query={"query": {"match_all": {}}},
+        index=ElasticSearchIndex.logo,
+        source=False,
     )
+    return set(int(item["_id"]) for item in scan_iter)
 
-    if not r.ok:
-        logger.log(
-            logging.INFO if r.status_code >= 500 else logging.WARNING,
-            "error while fetching stored logo IDs (%s): %s",
-            r.status_code,
-            r.text,
+
+def add_logos_to_ann(logo_embeddings: list[LogoEmbedding]) -> None:
+    """Index logo embeddings in Elasticsearch ANN index."""
+    es_client = get_es_client()
+
+    for logo_embedding in logo_embeddings:
+        embedding = np.frombuffer(logo_embedding.embedding, dtype=np.float32)
+        es_client.index(
+            index=ElasticSearchIndex.logo,
+            id=logo_embedding.logo_id,
+            document={
+                "embedding": embedding / np.linalg.norm(embedding),
+            },
         )
-        return set()
-
-    return set(r.json()["stored"])
 
 
-def add_logos_to_ann(image: ImageModel, logos: list[LogoAnnotation]) -> int:
-    if not logos:
-        return 0
+def save_nearest_neighbors(logo_embeddings: list[LogoEmbedding]) -> None:
+    """Save nearest neighbors of a batch of logo embedding."""
+    es_client = get_es_client()
 
-    image_url = settings.OFF_IMAGE_BASE_URL + image.source_image
+    for logo_embedding in logo_embeddings:
+        results = knn_search(
+            es_client, logo_embedding.embedding, settings.K_NEAREST_NEIGHBORS
+        )
+        results = [item for item in results if item[0] != logo_embedding.logo_id][
+            : settings.K_NEAREST_NEIGHBORS
+        ]
 
-    data = {
-        "image_url": image_url,
-        "logos": [{"bounding_box": logo.bounding_box, "id": logo.id} for logo in logos],
-    }
-    r = http_session.post(
-        settings.BaseURLProvider().robotoff().get() + "/api/v1/ann/add",
-        json=data,
-        timeout=30,
-    )
-
-    r.raise_for_status()
-    return r.json()["added"]
-
-
-def save_nearest_neighbors(logos: list[LogoAnnotation]) -> int:
-    logo_ids_params = ",".join((str(logo.id) for logo in logos))
-    r = http_session.get(
-        settings.BaseURLProvider().robotoff().get()
-        + "/api/v1/ann/batch?logo_ids="
-        + logo_ids_params,
-        timeout=30,
-    )
-
-    r.raise_for_status()
-    response = r.json()
-
-    results = {int(key): value for key, value in response["results"].items()}
-
-    logo_id_to_logo = {logo.id: logo for logo in logos}
-    missing_logo_ids = set(logo_id_to_logo.keys()).difference(set(results.keys()))
-
-    if missing_logo_ids:
-        logger.info("Missing logo IDs in response: %s", missing_logo_ids)
-
-    saved = 0
-    for logo_id, logo_results in results.items():
-        if logo_id in logo_id_to_logo:
-            logo = logo_id_to_logo[logo_id]
-            distances = [n["distance"] for n in logo_results]
-            logo_ids = [n["logo_id"] for n in logo_results]
-            logo.nearest_neighbors = {
+        if results:
+            logo_ids, distances = zip(*results)
+            logo_embedding.logo.nearest_neighbors = {
                 "distances": distances,
                 "logo_ids": logo_ids,
+                "updated_at": datetime.datetime.utcnow().isoformat(),
             }
-            logo.save()
-            saved += 1
+            logo_embedding.logo.save()
 
-    return saved
+
+def knn_search(
+    client: elasticsearch.Elasticsearch,
+    embedding_bytes: bytes,
+    k: int = settings.K_NEAREST_NEIGHBORS,
+) -> list[tuple[int, float]]:
+    embedding = np.frombuffer(embedding_bytes, dtype=np.float32)
+    knn_body = {
+        "field": "embedding",
+        "query_vector": embedding / np.linalg.norm(embedding),
+        "k": k + 1,
+        "num_candidates": k + 1,
+    }
+
+    results = client.search(
+        index=ElasticSearchIndex.logo, knn=knn_body, source=False, size=k + 1
+    )
+    if hits := results["hits"]["hits"]:
+        return [(int(hit["_id"]), 1.0 - hit["_score"]) for hit in hits]
+
+    return []
 
 
 @cachetools.cached(cachetools.LRUCache(maxsize=1))
@@ -315,8 +315,7 @@ def import_logo_insights(
 
 
 def generate_insights_from_annotated_logos(
-    logos: list[LogoAnnotation],
-    server_domain: str,
+    logos: list[LogoAnnotation], server_domain: str
 ) -> int:
     predictions = []
     for logo in logos:
@@ -349,8 +348,7 @@ def generate_insights_from_annotated_logos(
 
 
 def predict_logo_predictions(
-    logos: list[LogoAnnotation],
-    logo_probs: list[dict[LogoLabelType, float]],
+    logos: list[LogoAnnotation], logo_probs: list[dict[LogoLabelType, float]]
 ) -> list[Prediction]:
     predictions = []
 
