@@ -2,8 +2,8 @@ import datetime
 import pathlib
 from typing import Optional
 
-from requests.exceptions import ConnectionError as RequestConnectionError
-from requests.exceptions import HTTPError, Timeout
+import elasticsearch
+from PIL import Image
 
 from robotoff.insights.extraction import (
     DEFAULT_OCR_PREDICTION_TYPES,
@@ -22,6 +22,7 @@ from robotoff.models import (
     ImageModel,
     ImagePrediction,
     LogoAnnotation,
+    LogoEmbedding,
     Prediction,
     db,
     with_db,
@@ -29,6 +30,7 @@ from robotoff.models import (
 from robotoff.off import get_server_type, get_source_from_url
 from robotoff.products import Product, get_product_store
 from robotoff.slack import NotifierFactory
+from robotoff.triton import generate_clip_embedding
 from robotoff.types import ObjectDetectionModel, PredictionType
 from robotoff.utils import get_image_from_url, get_logger, http_session
 from robotoff.workers.queues import enqueue_job, high_queue
@@ -73,6 +75,7 @@ def run_import_image_job(
     enqueue_job(
         import_insights_from_image,
         high_queue,
+        job_kwargs={"result_ttl": 0},
         barcode=barcode,
         image_url=image_url,
         ocr_url=ocr_url,
@@ -85,6 +88,7 @@ def run_import_image_job(
     enqueue_job(
         run_logo_object_detection,
         high_queue,
+        job_kwargs={"result_ttl": 0},
         barcode=barcode,
         image_url=image_url,
         server_domain=server_domain,
@@ -92,6 +96,7 @@ def run_import_image_job(
     enqueue_job(
         run_nutrition_table_object_detection,
         high_queue,
+        job_kwargs={"result_ttl": 0},
         barcode=barcode,
         image_url=image_url,
         server_domain=server_domain,
@@ -122,6 +127,7 @@ def import_insights_from_image(
         enqueue_job(
             run_nutriscore_object_detection,
             high_queue,
+            job_kwargs={"result_ttl": 0},
             barcode=barcode,
             image_url=image_url,
             server_domain=server_domain,
@@ -228,9 +234,12 @@ def run_nutrition_table_object_detection(
     source_image = get_source_from_url(image_url)
 
     with db:
-        run_object_detection_model(
-            ObjectDetectionModel.nutrition_table, image, source_image
-        )
+        if image_model := ImageModel.get_or_none(source_image=source_image):
+            run_object_detection_model(
+                ObjectDetectionModel.nutrition_table, image, image_model
+            )
+        else:
+            logger.info("Missing image in DB for image %s", source_image)
 
 
 NUTRISCORE_LABELS = {
@@ -257,11 +266,16 @@ def run_nutriscore_object_detection(barcode: str, image_url: str, server_domain:
     source_image = get_source_from_url(image_url)
 
     with db:
+        if (image_model := ImageModel.get_or_none(source_image=source_image)) is None:
+            logger.info("Missing image in DB for image %s", source_image)
+            return
+
         image_prediction = run_object_detection_model(
-            ObjectDetectionModel.nutriscore, image, source_image
+            ObjectDetectionModel.nutriscore, image, image_model
         )
 
-    if not image_prediction:
+    if image_prediction is None:
+        # an ImagePrediction already exist
         return
 
     results = [
@@ -322,64 +336,102 @@ def run_logo_object_detection(
     source_image = get_source_from_url(image_url)
 
     with db:
-        image_prediction = run_object_detection_model(
-            ObjectDetectionModel.universal_logo_detector, image, source_image
-        )
-
-        if image_prediction is None:
-            # Can occur in normal conditions if an image prediction
-            # already exists for this image and model
+        if (image_model := ImageModel.get_or_none(source_image=source_image)) is None:
+            logger.info("Missing image in DB for image %s", source_image)
             return
 
-        logo_ids = []
-        for i, item in filter_logos(
-            image_prediction.data["objects"],
-            score_threshold=0.5,
-            iou_threshold=0.95,
-        ):
-            logo_ids.append(
-                LogoAnnotation.create(
-                    image_prediction=image_prediction,
-                    index=i,
-                    score=item["score"],
-                    bounding_box=item["bounding_box"],
-                ).id
-            )
+        image_prediction: ImagePrediction = run_object_detection_model(  # type: ignore
+            ObjectDetectionModel.universal_logo_detector,
+            image,
+            image_model,
+            return_null_if_exist=False,
+        )
+        existing_logos = list(image_prediction.logos)
 
-    logger.info("%s logos found for image %s", len(logo_ids), source_image)
-    if logo_ids and process_logos:
+        if not existing_logos:
+            logos: list[LogoAnnotation] = []
+            for i, item in filter_logos(
+                image_prediction.data["objects"],
+                score_threshold=0.5,
+                iou_threshold=0.95,
+            ):
+                logos.append(
+                    LogoAnnotation.create(
+                        image_prediction=image_prediction,
+                        index=i,
+                        score=item["score"],
+                        bounding_box=item["bounding_box"],
+                    )
+                )
+            logger.info("%s logos found for image %s", len(logos), source_image)
+        else:
+            # Logos already exist for this image prediction, we just make sure embeddings
+            # are in DB as well
+            logos = [
+                existing_logo
+                for existing_logo in existing_logos
+                if list(existing_logo.embeddings) == 0
+            ]
+
+    if logos and process_logos:
+        with db:
+            save_logo_embeddings(logos, image)
         enqueue_job(
             process_created_logos,
             high_queue,
-            job_kwargs={},
+            job_kwargs={"result_ttl": 0},
             image_prediction_id=image_prediction.id,
             server_domain=server_domain,
         )
 
 
+def save_logo_embeddings(logos: list[LogoAnnotation], image: Image.Image):
+    """Generate logo embeddings using CLIP model and save them in
+    logo_embedding table."""
+    cropped_images = []
+    for logo in logos:
+        y_min, x_min, y_max, x_max = logo.bounding_box
+        (left, right, top, bottom) = (
+            x_min * image.width,
+            x_max * image.width,
+            y_min * image.height,
+            y_max * image.height,
+        )
+        cropped_images.append(image.crop((left, top, right, bottom)))
+    embeddings = generate_clip_embedding(cropped_images)
+
+    with db:
+        for i in range(len(logos)):
+            logo_id = logos[i].id
+            logo_embedding = embeddings[i]
+            LogoEmbedding.create(logo_id=logo_id, embedding=logo_embedding.tobytes())
+
+
 @with_db
 def process_created_logos(image_prediction_id: int, server_domain: str):
-    logos = (
-        LogoAnnotation.select()
+    logo_embeddings = list(
+        LogoEmbedding.select()
+        .join(LogoAnnotation)
         .join(ImagePrediction)
-        .join(ImageModel)
         .where(ImagePrediction.id == image_prediction_id)
     )
 
-    if logos:
-        image_instance = logos[0].image_prediction.image
+    if not logo_embeddings:
+        return
 
-        try:
-            add_logos_to_ann(image_instance, logos)
-        except (RequestConnectionError, HTTPError, Timeout) as e:
-            logger.info("Request error during logo addition to ANN", exc_info=e)
-            return
+    try:
+        add_logos_to_ann(logo_embeddings)
+    except (elasticsearch.ConnectionError, elasticsearch.ConnectionTimeout) as e:
+        logger.info("Request error during logo addition to ANN", exc_info=e)
+        return
 
-        try:
-            save_nearest_neighbors(logos)
-        except (RequestConnectionError, HTTPError, Timeout) as e:
-            logger.info("Request error during ANN batch query", exc_info=e)
-            return
+    try:
+        save_nearest_neighbors(logo_embeddings)
+    except (elasticsearch.ConnectionError, elasticsearch.ConnectionTimeout) as e:
+        logger.info("Request error during ANN batch query", exc_info=e)
+        return
 
-        thresholds = get_logo_confidence_thresholds()
-        import_logo_insights(logos, thresholds=thresholds, server_domain=server_domain)
+    logos = [embedding.logo for embedding in logo_embeddings]
+
+    thresholds = get_logo_confidence_thresholds()
+    import_logo_insights(logos, thresholds=thresholds, server_domain=server_domain)
